/*
	La version toujours plus dveloppe de l'applet "maison"
	Possibilit de slectionner des figures et surtout de modifier
	leur taille avec la souris.

	(C) J. Ferber - Octobre 2000
*/

package maison;

import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import java.io.*;


/** La classe MaisonApplet qui gre les interactions avec l'utilisateur.
*/
public class MaisonApp extends JFrame implements ActionListener, Runnable
{

        JPanel figPanel; // le panel qui contient tous les boutons
                           // pour les figures
    	DrawingPane drawPane;


	public MaisonApp() {
		setTitle("Untitled");
		getContentPane().setLayout(new BorderLayout());
		JPanel p = new JPanel();
		getContentPane().add(p,"North");

		// les boutons
		addButton(p,"Nouveau","Nouveau dessin","/icons/New24.gif");
		addButton(p,"Sauver","sauver","/icons/Save24.gif");
		addButton(p,"Sauver sous","sauver sous","/icons/SaveAs24.gif");
		addButton(p,"Ouvrir","ouvrir un fichier","/icons/Open24.gif");

		addButton(p,"Select","selectionner une figure","/icons/Select24.gif");
		addButton(p,"Grille","afficher, supprimer la grille",null);
		addButton(p,"Couleur","Modifier la couleur",null);
		addButton(p,"Run","Dmarre l'animation",null);
		addButton(p,"Stop","Arrte l'animation","/icons/Stop24.gif");

		// les menus
		JMenuBar menubar=new JMenuBar();
        setJMenuBar(menubar);	// on installe le menu bar
		JMenu menuFile=new JMenu("File"); // on installe le premier menu
        menubar.add(menuFile);

        addMenuItem(menuFile, "Nouveau", "Nouveau", KeyEvent.VK_N);
        addMenuItem(menuFile, "Ouvrir", "Ouvrir", -1);
        addMenuItem(menuFile, "Sauver", "Sauver", KeyEvent.VK_S);
        addMenuItem(menuFile, "Sauver sous", "Sauver sous", -1);


		JMenu menuAnimation=new JMenu("Animation"); // on installe le premier menu
        menubar.add(menuAnimation);
        addMenuItem(menuAnimation, "Run", "Run", -1);
        addMenuItem(menuAnimation, "Stop", "Stop", -1);

		JMenu menuHelp=new JMenu("Aide"); // on installe le premier menu
        menubar.add(menuHelp);
        addMenuItem(menuHelp, "Aide", "Help", -1);
        addMenuItem(menuHelp, "A propos", "About", -1);

		figPanel = new JPanel(new GridLayout(4,1));
		getContentPane().add(figPanel,"West");

  		drawPane=new DrawingPane();
    	getContentPane().add(drawPane,"Center");
		drawPane.setPreferredSize(new Dimension(600,400));

        installFigure("Rectangle");
        installFigure("Oval");
        installFigure("Ligne");
        installFigure("Maison");

		// pour sortir de l'application
		addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

		pack();
		show();

	}

	public static void main(String[] args){
		new MaisonApp();
	}

	// l'animation

    int delay = 30;

  	Thread process=null;
	protected boolean running;
	long iteration=0;

	/* la methode qui dclenche l'animation
	**/
	void animate(){
	  if (!running){
		iteration=0;
	  	running = true;
      	process = new Thread(this);
      	// process.setPriority(Thread.MAX_PRIORITY);
      	process.start();
      }
    }

    public void stop(){
		running = false;
		drawPane.setMode(DrawingPane.MODE_SELECT);
	}

    public void run()
    {
		drawPane.setMode(DrawingPane.MODE_SELECT);
      while (running) {
		// if (showIteration)
		//	System.out.println("iteration: " + iteration);
		drawPane.runFigures();
      	repaint();

		iteration++;

	  	try {process.sleep(delay);}
		catch (InterruptedException e) 	{
			running = false; break;
		}
	  }
    }

	void clear(){
		setTitle("Untitled");
		fileName=null;
		dirName=null;
		drawPane.setMode(DrawingPane.MODE_SELECT);
	}

	public void actionPerformed(ActionEvent e) {
		String c = e.getActionCommand();

		if (c.equals("Select")){
      		drawPane.setMode(DrawingPane.MODE_SELECT);
  		} else if (c.equals("Couleur")) {
        	drawPane.incrColor();
  		} else if (c.equals("Grille")) {
			drawPane.toggleGrid();
		} else if (c.equals("Nouveau")) {
			clear();
			drawPane.clear();
		} else if (c.equals("Run")) {
			animate();
		} else if (c.equals("Stop")) {
			stop();
		} else if (c.equals("Ouvrir"))
			openFile();
		else if (c.equals("Sauver"))
			save();
		else if (c.equals("Sauver sous"))
			saveAs();
	}

        void installFigure(String name){
            JButton b = new JButton(name);
            figPanel.add(b);
            b.addActionListener(new ActionListener(){
                  public void actionPerformed(ActionEvent ev){
					  drawPane.unselect(true);
                      drawPane.createFigure(ev.getActionCommand());

                  }
              });
        }

	String dirName;
	String fileName;

	void openFile(){
		if (getFileDialog())
		   loadFile();
		drawPane.repaint();
	}


	boolean getFileDialog(){

   		FileDialog f = new FileDialog(this,"Lecture de figures",FileDialog.LOAD);
       	if (dirName != null)
				f.setDirectory(dirName);
		else
				f.setDirectory(System.getProperty("user.dir",null));
       	f.show();
       	fileName = f.getFile();
		if (fileName != null){
       		dirName = f.getDirectory();
			return(true);
		}
		else
			return(false);
	}


	void loadFile()
	{
       String fname;

       if (fileName != null){
		System.out.println(">>chargement de "+dirName+fileName);
       	try{
			BufferedReader r = new BufferedReader(
                           	new FileReader(dirName+fileName));

   			StreamTokenizer st = new StreamTokenizer(r);
			st.whitespaceChars((int) '!', (int) '(');
            st.whitespaceChars((int) '*', (int) '-');
            st.whitespaceChars((int) ':', (int) '?');
            st.wordChars((int) '[', (int) ']');
			st.eolIsSignificant(false);
			st.slashSlashComments(true);
			st.quoteChar((int)'"');
			st.quoteChar((int)'');

			drawPane.parse(st);
			setTitle(fileName);
			r.close();
		 } catch (FileNotFoundException e) {
            	System.err.println("openFile: " + e);
		 } catch (IOException e) {
            	System.err.println("openFile: " + e);
		 }
	   }
	}

	void saveAs() {
   		FileDialog f = new FileDialog(this,"Sauvegarde de figures",FileDialog.SAVE);
       	if (dirName != null)
				f.setDirectory(dirName);
		else
				f.setDirectory(System.getProperty("user.dir",null));
       if (fileName != null) f.setFile(fileName);
       f.show();
       dirName = f.getDirectory();
       fileName = f.getFile();
       save1();

   }

	void save() {
       if (fileName == null){
       		FileDialog f = new FileDialog(this,"Sauvegarde de figures",FileDialog.SAVE);
       		if (dirName != null)
				f.setDirectory(dirName);
			else
				f.setDirectory(System.getProperty("user.dir",null));
       		if (fileName != null) f.setFile(fileName);
       		f.show();
       		dirName = f.getDirectory();
       		fileName = f.getFile();
       }
       save1();
   }

   void save1() {
   	if (fileName != null) {
       	try{
           	System.out.println(":: sauvegarde de " + dirName + fileName);
           	PrintWriter outFile = new PrintWriter(new FileWriter(dirName+fileName));
            drawPane.save(outFile);
            outFile.close();
        	} catch (FileNotFoundException e) {
            	System.err.println("saveContent: " + e);
        	} catch (IOException e) {
            	System.err.println("saveContent: " + e);
           }
       System.out.println(":: fichier " + fileName + " sauve");
       }
   }

   	void addButton(JPanel p, String name, String tooltiptext, String imageName) {
	  JButton b;
	  if ((imageName == null) || (imageName.equals(""))) {
              b = (JButton) p.add(new JButton(name));
	  }
	  else {
		   java.net.URL u = this.getClass().getResource(imageName);
	       if (u != null){
			  ImageIcon im = new ImageIcon (u);

			  b = (JButton) p.add(new JButton(im));
		   } else
				  b = (JButton) p.add(new JButton(name));
		   b.setActionCommand(name);
	  }

	  b.setToolTipText(tooltiptext);
	  b.setMargin(new Insets(0,0,0,0));
	  b.addActionListener(this);
	}

	void addMenuItem(JMenu m, String label, String command, int key) {
        JMenuItem menuItem;
	    menuItem = new JMenuItem(label);
        m.add(menuItem);

        menuItem.setActionCommand(command);
        menuItem.addActionListener(this);
		if (key > 0) {
			if (key != KeyEvent.VK_DELETE)
				menuItem.setAccelerator(KeyStroke.getKeyStroke(key, Event.CTRL_MASK, false));
			else
				menuItem.setAccelerator(KeyStroke.getKeyStroke(key, 0, false));
		}
	}

}
